# Diverging bars
# from http://r-statistics.co
# Recall that to use a geom_bar as a bar chart instead of just a univariate histogram,
# you have to use stat=identity, and also provide both an x and y in the aes.
# The x should be a character or factor.  The y is numeric.
# In order to make sure you get diverging bars instead of just bars, make sure
# your categorical (factor) variable has 2 categories that change value at a certain 
# threshold of the continuous variable.  In the below example, the mpg from mtcars
# is normalized by computing the z score.  Those vehicles with mpg above zero
# are marked green,and those below are marked red.

library(ggplot2)
theme_set(theme_bw())  
mtcars

# Data Prep
data("mtcars")  # load data
mtcars$`car name` <- rownames(mtcars)  # create new column for car names
mtcars$mpg_z <- round((mtcars$mpg - mean(mtcars$mpg))/sd(mtcars$mpg), 2)  # compute normalized mpg
# why z scores?  A positive z score means "above the mean".  Negative means "below the mean".
# You could accomplish the same thing by computing the mean, and the comparing against that.

mtcars$mpg_type <- ifelse(mtcars$mpg_z < 0, "below", "above")  # above / below avg flag
head(mtcars)    # let's look at our new variables

mtcars <- mtcars[order(mtcars$mpg_z), ]  # sort
head(mtcars, 20)
class(mtcars$'car name')

mtcars$`car name` <- factor(mtcars$`car name`, levels = mtcars$`car name`)  # convert to factor to retain sorted order in plot.
# That was all setup--creating the variables that you need in order to plot.
# Below is where we construct the plot.

# Diverging Barcharts
ggplot(mtcars, aes(x=`car name`, y=mpg_z)) + 
  geom_bar(stat='identity', aes(fill=mpg_type), width=.5)  +
  scale_fill_manual(name="Mileage", 
                    labels = c("Above Average", "Below Average"), 
                    values = c("above"="#00ba38", "below"="#f8766d")) + 
  labs(subtitle="Normalized mileage from 'mtcars'", 
       title= "Diverging Bars") + 
  coord_flip()


# Diverging Barcharts.  And this works the same too.  (Moved fill to ggplot)
ggplot(mtcars, aes(x=`car name`, y=mpg_z, fill = mpg_type)) + 
  geom_bar(stat='identity',  width=.5)  +
  scale_fill_manual(name="Mileage", 
                    labels = c("Above Average", "Below Average"), 
                    values = c("above"="#00ba38", "below"="#f8766d")) + 
  labs(subtitle="Normalized mileage from 'mtcars'", 
       title= "Diverging Bars") + 
  coord_flip()

# Diverging Barcharts.  Let's add an mpg label at the end of each bar
# Challenge: horizontal justify should be different for green and red bars!!
# So need to create another field with the amount of justification.

ggplot(mtcars, aes(x=`car name`, y=mpg_z, fill = mpg_type)) + 
  geom_bar(stat='identity',  width=.5)  +
  geom_text(aes(label=mpg), size=3, hjust = 1)+
  scale_fill_manual(name="Mileage", 
                    labels = c("Above Average", "Below Average"), 
                    values = c("above"="#00ba38", "below"="#f8766d")) + 
  labs(subtitle="Normalized mileage from 'mtcars'", 
       title= "Diverging Bars") + 
  coord_flip()

# But that works well for the red bars, but not the green!
mtcars$justify <- ifelse(mtcars$mpg_z < 0, 1.2,-0.2)  # horizontal justification
head(mtcars)
tail(mtcars)

ggplot(mtcars, aes(x=`car name`, y=mpg_z, fill = mpg_type)) + 
  geom_bar(stat='identity',  width=.5)  +
  geom_text(aes(label=mpg, hjust = justify), vjust = .3, size=2.5)+
  scale_fill_manual(name="Mileage", 
                    labels = c("Above Average", "Below Average"), 
                    values = c("above"="#00ba38", "below"="#f8766d")) + 
  labs(subtitle="Normalized mileage from 'mtcars'", 
       title= "Diverging Bars") + 
  coord_flip()
